c====================================================
      program j1j2
c====================================================

      implicit none

      real j1,j2              ! Parameters
      parameter(j1=1.,j2=0.5)   !

      integer n                 ! Lattice size (1D)
      parameter(n=24)           !

      integer m                 ! # of spins up
      parameter(m=12)            ! Sz=0.5*(2m-n)

      integer nsym              ! # of symmetries
      parameter(nsym=n)        ! 1D ring: nsym=n translations
      
      integer nparmax           ! Upper limit on # of "parents"
      parameter(nparmax=120000)     ! nparmax ~ C(n,m)/nsym + 10

      integer niter             ! Iteration #
      parameter(niter=70)      !

      integer nparents          ! # of parents (at run-time)

      integer parents           ! List of parents (coded as integers)
      dimension parents(nparmax)

      integer degen             ! List of degenerescies of
      dimension degen(nparmax)  ! parents

      integer sector           ! Label of symmetry sector
      parameter(sector=1)      ! q=0,..,2pi => sector=1,...,size+1

      real diag               ! Hamiltonien
      integer nondiag
      dimension diag(nparmax)
      dimension nondiag(nparmax,2*n)


      real diago              ! Diagonal part of tridiag matrix
      dimension diago(0:niter)  !

      real  ndiago             ! Non diag part of tridiag matrix
      dimension ndiago(niter)   !

      real  d                  ! Diagonal part of tridiag matrix
      dimension d(niter)        !

      real  e                  ! Non diag of H tridiagonal
      dimension e(niter)        !

      real  z                  !
      dimension z(niter,niter)  ! Matrix for tqlmod 



      integer i,j,l,p,k             ! Loop indices
      integer i1,i2,i3,i4,i5,i6 !

      integer conf              ! Running config
 

      integer bits1             ! List of site numbers with a bit eq to 1
      dimension bits1(m)        ! 

      real  vec1,vec2,vec3,hvec2,vecnul
      dimension vec1(nparmax,2)
      dimension vec2(nparmax,2)
      dimension vec3(nparmax,2)
      dimension hvec2(nparmax,2)
      dimension vecnul(nparmax,2)

      real  a,b,normc1,normc2,normc3

      real seed               ! Needed for random #
      real random,start       ! generator

      integer ilim              ! Max integer to code n bits
      parameter(ilim=2**n-1)


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                         c
c                                 MAIN                                    c
c                                                                         c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      conf=0                    !
                                !
      do i=1,nparmax            ! 
         parents(i)=0           !
      enddo                     !
      nparents=0                !
                                !
      do i=0,niter              !
         diago(i)=0.            !
      enddo                     !
                                ! Initialise variables
      do i=1,niter              !
         d(i)=0.                !
         ndiago(i)=0.           !
         e(i)=0.                !
         do j=1,niter           !
            z(j,i)=0.           !
         enddo                  !
         z(i,i)=1.              !
      enddo                     !
                                !
      do j=1,2                  !
         do i=1,nparmax         !
            vec1(i,j)=0.        !
            vec2(i,j)=0.        !
            vec3(i,j)=0.        !
            hvec2(i,j)=0.       !
            vecnul(i,j)=0.      !
         enddo                  !
      enddo                     !


      p=0
c Select integers with the right # of spins up:
      do i=1,ilim
         call popcnt(n,i,k)  ! Count the number of bits '1' in integer i
         if (k.eq.m) then 
            p=p+1 
            conf=i
            call parentslist(conf,parents,nparents,nparmax
     &           ,nsym,n)
                             ! Check if conf is parent & add it to parent list 
                             ! if it is -- Return nparents
         endif      
      enddo
      print*,'Total number of states:',p
      print*,'Final # of parents:',nparents

      call eliminate_parents(parents,sector,degen,nparents,
     &     nsym,nparmax,n)
                                ! Eliminate parents /
                                ! sum of characters of sym keeping it
                                ! invariant = 0
                                ! Calculate degenerecy of parent
                                ! = # of sym that keep it invariant

      print*,'Actual # of parents in sector ',sector,' :',nparents


      call hamilt(diag,nondiag,j1,j2,parents,
     &     nparents,nparmax,nsym,n)

                                ! Construct H
	
      seed=-2.d0                                    !
      normc1=0.                                    ! 
      start=random(seed)                            !
      do i=1,nparents                               !
         vec1(i,1)=random(seed)                     !
         vec1(i,2)=random(seed)                     ! 
         normc1=normc1+vec1(i,1)**2+vec1(i,2)**2  ! 
      end do                                        ! 
      normc1=sqrt(normc1)                         ! 
      do i=1,nparents                               ! Calculate a random
         vec1(i,1)=vec1(i,1)/normc1                !    |0> =vec1
         vec1(i,2)=vec1(i,2)/normc1                !
      end do                                        !
                                                    !
      do j=1,2                                      !
         do i=nparents+1,nparmax                    !
            vec1(i,j)=0.                            !
         enddo                                      !
      enddo                                         !


      call hpsi(vec1,hvec2,diag,nondiag,degen,j1,j2,     ! Calculate H|0>
     &     sector,nparents,nparmax,n)                   !    =hvec2
      
      call lanczos(vecnul,1.,vec1,1.,hvec2,vec2,         ! Calculate  |1>
     &     normc2,a,b,nparents,nparmax)                 !    =vec2
         
      diago(0)=a
      normc1=1.
      
      do i=1,niter-1
         call hpsi(vec2,hvec2,diag,nondiag,degen,j1,j2,   ! Calculate H|i>
     &        sector,nparents,nparmax,n)                 !   =hvec2
      

         call lanczos(vec1,normc1,vec2,normc2,
     &        hvec2,vec3,normc3,a,b,nparents,nparmax)    ! Calculate |i+1>
                                                          !   =vec3

         diago(i)=a
         ndiago(i)=b

         do j=1,2                                         !
            do l=1,nparents                               !
               vec1(l,j)=vec2(l,j)                        ! Permutation
               vec2(l,j)=vec3(l,j)                        ! 2 -> 1
            enddo                                         ! 3 -> 2
         enddo                                            !
                                                          !
         normc1=normc2                                  !
         normc2=normc3                                  ! 


         do j=1,i                               !
            d(j)=diago(j-1)                     ! Diagonal part of H tridiag
         enddo                                  !

         do j=2,i                               !
            e(j)=ndiago(j-1)                    ! Upper off-diag line of
         enddo                                  ! H tridiagonal


         do l=1,niter
            do j=1,niter
               z(j,l)=0.
            enddo
            z(l,l)=1.
         enddo

         call tqlmod(d,e,i,niter,z)             ! Diagonalize tridiag matrix
         if ((i+1).gt.1) then
c            print*,'iter',i+1,'   ',(d(j),j=1,3)
            print*,'iter',i+1,'   ',(d(j)/float(n),j=1,3)
         endif
      enddo
      end


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                         c
c                         FUNCTIONS & SUBROUTINES                        c
c                                                                         c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine parentslist(conf,parents,nparents,nparmax,
     &     nsym,size) 
                                           ! Construct the list of parents
      implicit none                        ! by adding conf if conf is a parent
      integer conf,parents,nparents,nparmax 
      dimension parents(nparmax)
      integer i,nsym,size,symmetric

      do i=1,nsym
         if ((symmetric(conf,i,size)).lt.conf) then
            goto 1
         else
            continue
         endif
      enddo
      nparents=nparents+1
      parents(nparents)=conf
 1    end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function smallestsym(conf,nsym,size,indsym)
                                ! Compute the parent of conf (defined as
                                ! smallest integer amoung the nsym states)
                                ! on a ring of size size
                                ! indsym is the symm #
                                ! relating conf to smallestsym
                                ! ( the no of id is 1)
      implicit none
      integer conf,smallestsym,nsym,symmetric
      integer i,size,indsym,smallest

      smallestsym=conf
      indsym=1
      do i=0,nsym-1
         smallest=symmetric(conf,i,size)
         if (smallest.lt.smallestsym) then
            indsym=i+1
            smallestsym=smallest
         endif
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function permutation(conf,j,k) ! Permutation of j & k bits
      implicit none
      integer permutation
      integer conf,j,k

      permutation=conf
      call mvbits(conf,j,1,permutation,k)
      call mvbits(conf,k,1,permutation,j)

      end

      


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      subroutine printbits(i,nbits)     ! Print out the nbits first bits
                                        ! of integer i
      implicit none
      integer nbits,i
      integer j

      print*,nbits,' Binary coding of integer ',i,':'
      do j=nbits-1,0,-1
         print*,ibits(i,j,1)
      enddo
      print*,'***********************'
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      function symmetric(conf,trans,size) ! Conf translated by length 'trans'
      implicit none                       ! Output back into 'conf'
      integer symmetric                     ! Lattice of size size
      integer conf,trans,size

      symmetric=ishftc(conf,trans,size)
      end

c     OTHER VERSION
c      function symmetric(conf,trans,size) 
c                                          
c      implicit none                       
c      integer symmetric              
c      integer conf,trans,size
c      integer bloc
c
c      bloc=ibits(conf,size-trans,trans)
c      symmetric=(2**trans)*conf+bloc-(2**size)*bloc
c      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     ***** NOT USED HERE *****

      subroutine makeconf(conf,bits,nbits)   ! Construct conf from the list
                                             ! of the 'nbits' sites with 
                                             ! bits '1' (provided in 'bits')
      implicit none
      integer conf,bits,nbits
      dimension bits(nbits)
      integer i

      conf=0
      do i=1,nbits
         conf=conf+2**(bits(i)-1)
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function character(trans,sector,size)
      implicit none
      complex character
      integer trans, sector, size
      real  pi,reel,imag

      pi=acos(-1.)
      reel=cos(2.*pi*float(trans)*(float(sector)-1.)/float(size))
      imag=sin(2.*pi*float(trans)*(float(sector)-1.)/float(size))
      character=cmplx(reel,imag)
      end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      subroutine eliminate_parents(parents,sector,degen,
     &     nparents,nsym,nparmax,size)

      implicit none
      integer parents,sector,degen,nparents,nsym,nparmax
      dimension parents(nparmax),degen(nparmax)
      complex sumcarac,character
      dimension sumcarac(nparmax)
      integer i,j,size,symmetric

      do i=1,nparmax
         degen(i)=0
         sumcarac(i)=cmplx(0.,0.)
      enddo

      do i=1,nparents
         do j=1,nsym
            if (parents(i).eq.symmetric(parents(i),j-1,size)) then
               degen(i)=degen(i)+1
               sumcarac(i)=sumcarac(i)+character(j,sector,size)
            endif
         enddo
      enddo

      j=0
      do i=1,nparents
         if (cabs(sumcarac(i)).gt.1.e-4) then
            j=j+1
            parents(j)=parents(i)
            degen(j)=degen(i)
         endif
      enddo

      nparents=j

      do i=nparents+1,nparmax
         parents(i)=0
         degen(i)=1
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function address(conf,parents,nparents,nparmax,size)
      implicit none
      integer address
      integer conf,parents,nparents,nparmax,size
      dimension parents(nparmax)
      integer i,test,oldtest,oldoldtest

      test=int(float(nparents)/2.)
      oldtest=0
      oldoldtest=0
      i=1

      do while((parents(test).ne.conf).and.(test.ne.oldoldtest)
     &     .and.(test.le.nparents).and.(test.ge.1))
         i=i+1
         if (conf.lt.parents(test)) then
            oldoldtest=oldtest
            oldtest=test
            test=test-max(int(float(nparents)/(2.**i)),1)
         else
            oldoldtest=oldtest
            oldtest=test
            test=test+max(int(float(nparents)/(2.**i)),1)
            oldoldtest=oldtest
         endif
      enddo

      if (parents(test).eq.conf) then
         address=test
      else
         address=0
      endif

      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function neighbor(site,distance,size) ! provide the site # at a distance
                                          ! 'distance' of site 'site' on a ring
      implicit none
      integer neighbor
      integer site,distance,size

      if ((site+distance).le.size) then
         neighbor=site+distance
      else
         neighbor=site+distance-size
      endif
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine hamilt(diag,nondiag,j1,j2,parents,
     &     nparents,nparmax,nsym,size)
      implicit none

      integer parents,nparents,nparmax,size,column
      integer smallestsym,nsym,indsym,permutation,address
      dimension parents(nparmax)
      integer i,j,k,adr,smallest,neighbor,ij
      real  diag,j1,j2
      dimension diag(nparmax)
      integer nondiag
      dimension nondiag(nparmax,2*size)
      column=0


      do i=1,2*size
         do j=1,nparmax
            nondiag(j,i)=0
         enddo
      enddo

      do i=1,nparmax
         diag(i)=0.
      enddo

      do i=1,nparents
         column=0
         do ij=0,1
            do j=1,size
               k=neighbor(j,1+ij,size)
               if (ibits(parents(i),k-1,1)+
     &              ibits(parents(i),j-1,1).eq.1) then
                  diag(i)=diag(i)-(j1*(1.-float(ij))+j2*float(ij))/4.
                  column=column+1
                  smallest=smallestsym(permutation(parents(i),j-1,k-1)
     &                 ,nsym,size,indsym)
                  adr=address(smallest,parents,nparents,nparmax,size)
                  if (adr.ne.0) then
                     call mvbits(adr,0,20,nondiag(i,column),0)
                     call mvbits(indsym,0,5,nondiag(i,column),25)
                     call mvbits(ij,0,1,nondiag(i,column),30)
                  endif   
               else
                  diag(i)=diag(i)+(j1*(1.-float(ij))+j2*float(ij))/4.
               endif
            enddo
         enddo
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      subroutine hpsi(vec1,vec2,diag,nondiag,degen,   ! Calculate H on vec1
     &     j1,j2,sector,nparents,nparmax,size)     
      implicit none                                   ! |vec2>=H|vec1>
      real  vec1,vec2
      real  diag
      integer nondiag,degen
      real  j1,j2
      integer nparents,nparmax,size
      dimension vec1(nparmax,2),vec2(nparmax,2)
      dimension diag(nparmax),degen(nparmax)
      dimension nondiag(nparmax,2*size)
      integer i,j,ij,addconf,indsym,sector
      complex character,carac

      do j=1,2                                        ! Diagonal part
         do i=1,nparents
            vec2(i,j)=diag(i)*vec1(i,j)
         enddo
      enddo

      do j=1,2*size                                 ! Off-diagonal part
         do i=1,nparents                              
            addconf=0
            indsym=0
            ij=0
            call mvbits(nondiag(i,j),0,20,addconf,0)
            if (addconf.ne.0) then
               call mvbits(nondiag(i,j),25,5,indsym,0)
               call mvbits(nondiag(i,j),30,1,ij,0)
               carac=character(indsym-1,sector,size)
               vec2(i,1)=vec2(i,1)
     &              +(real(carac)*vec1(addconf,1)
     &              -aimag(carac)*vec1(addconf,2))
     &              *sqrt((float(degen(addconf))
     &              /(float(degen(i)))))
     &              *(j1*(1.-float(ij))+j2*float(ij))*0.5
               vec2(i,2)=vec2(i,2)
     &              +(real(carac)*vec1(addconf,2)
     &              +aimag(carac)*vec1(addconf,1))
     &              *sqrt((float(degen(addconf))
     &              /(float(degen(i)))))
     &              *(j1*(1.-float(ij))+j2*float(ij))*0.5
            endif
         enddo
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine lanczos(vec1,normc1,vec2,normc2,hvec2,vec3,
     &     normc3,a,b,nparents,nparmax) 
                                                  ! vec3  & normc3 generated
      implicit none                               ! by 'lanczos' from
      integer nparents,nparmax                    !  vec2,vec1
      real  vec1,vec2,vec3,hvec2                ! and  |vec2>=hvec2
      dimension vec1(nparmax,2),vec2(nparmax,2)
      dimension vec3(nparmax,2),hvec2(nparmax,2)
      real  normc1,normc2,normc3,a,b
      integer i,j


      a=0.                                               !   
      do i=1,nparents                                    !
         a=a+(hvec2(i,1)*vec2(i,1)+hvec2(i,2)*vec2(i,2)) !
      enddo                                              !
                
      b=sqrt(normc2)

      normc3=0.
      do j=1,2
         do i=1,nparents
            vec3(i,j)=hvec2(i,j)
     &           -a*vec2(i,j)
     &           -b*vec1(i,j)
            normc3=normc3+vec3(i,j)**2
         enddo
      enddo

      do j=1,2
         do i=1,nparents
            vec3(i,j)=vec3(i,j)/sqrt(normc3)
         enddo
      enddo
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c         Subroutine TQLI.F (Diagonalisation of tridiagonal matrix)
c         Eigenvalues (and corresponding eigenvectors) are ordered 
c         in increasing order
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE tqlmod(d,e,n,np,z)
      INTEGER n,np
      REAL d(np),e(np),z(np,np)
CU    USES pythag
      INTEGER i,iter,k,l,m
      REAL b,c,dd,f,g,p,r,s,pythag
      do 11 i=2,n
        e(i-1)=e(i)
11    continue
      e(n)=0.
      do 15 l=1,n
        iter=0
1       do 12 m=l,n-1
          dd=abs(d(m))+abs(d(m+1))
          if (abs(e(m))+dd.eq.dd) goto 2
12      continue
        m=n
2       if(m.ne.l)then
c          if(iter.eq.30)pause 'too many iterations in tqli'
          iter=iter+1
          g=(d(l+1)-d(l))/(2.*e(l))
          r=pythag(g,1.)
          g=d(m)-d(l)+e(l)/(g+sign(r,g))
          s=1.
          c=1.
          p=0.
          do 14 i=m-1,l,-1
            f=s*e(i)
            b=c*e(i)
            r=pythag(f,g)
            e(i+1)=r
            if(r.eq.0.)then
              d(i+1)=d(i+1)-p
              e(m)=0.
              goto 1
            endif
            s=f/r
            c=g/r
            g=d(i+1)-p
            r=(d(i)-g)*s+2.*c*b
            p=s*r
            d(i+1)=g+p
            g=c*r-b
C     Omit lines from here ...
            do 13 k=1,n
              f=z(k,i+1)
              z(k,i+1)=s*z(k,i)+c*f
              z(k,i)=c*z(k,i)-s*f
13          continue
C     ... to here when finding only eigenvalues.
14        continue
          d(l)=d(l)-p
          e(l)=g
          e(m)=0.
          goto 1
        endif
15    continue

 	call piksrmod(n,d,z)	

      return
      END

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                  PYTHAG.F (necessary for tqli.f)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      FUNCTION pythag(a,b)
      REAL a,b,pythag
      REAL absa,absb
      absa=abs(a)
      absb=abs(b)
      if(absa.gt.absb)then
        pythag=absa*sqrt(1.+(absb/absa)**2)
      else
        if(absb.eq.0.)then
          pythag=0.
        else
          pythag=absb*sqrt(1.+(absa/absb)**2)
        endif
      endif
      return
      END

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                Sorting d et z
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE piksrmod(n,arr,brr)
      INTEGER n
      REAL arr(n),brr(n,n)
      INTEGER i,j,k
      REAL a,b(n)
      do 12 j=2,n
        a=arr(j)
	do k=1,n
        b(k)=brr(k,j)
      end do
        do 11 i=j-1,1,-1
          if(arr(i).le.a)goto 10
          arr(i+1)=arr(i)
	do k=1,n
          brr(k,i+1)=brr(k,i)
      end do    
11      continue
        i=0
10      arr(i+1)=a
	do k=1,n
        brr(k,i+1)=b(k)
      end do  
12    continue
      return
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc                  RANDOM.F
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 	
 	function random(a)
        implicit real (a-h,o-z)
      	parameter (ntab=128,ntab1=ntab-1,eps=1.d0-2.d-16)
      	dimension b(ntab)  
	save b
      	data b /NTAB*0.d0/
      	if(a.lt.0.d0)then
	  a=-a
	  ia=a
	  a=a-ia
	  b(1)=1.d0-a
	  do j=2,ntab
	    a=a*6185760.643692057d0+0.4142673443108168d0
     $	    +b(j-1)*178736.5675744203d0
	    ia=a
      	    a=a-ia
      	    bb=a*3141.592542102691d0+0.7321673443901507d0
     $      +b(j-1)*2718.073574407851d0
      	    ib=bb
      	    bb=bb-ib
      	    bb=a*177.9502390045814d0+0.5630259800256374d0
     $      +b(j-1)*891.71306987203691d0
      	    ib=bb
      	    b(j)=bb-ib	
      	  enddo
      	else
c       ** SEULE CETTE PARTIE EST REALISEE APRES INITIALISATION **  	
      	  j1=ntab1*a+1
      	  b1=b(j1)
      	  a=a*6180339.90391561234d0+0.4142136143235673d0
     $ 	  +b1*6931471.85675741376d0
	  ia=a
      	  a=a-ia
      	  b(j1)=1.d0-a
      	  
c      	  goto 1
	  j2=ntab1*b1+1
      	  b2=b(j2)
      	  a=a*3141.59254210379531d0+0.7724539443698741d0
     $ 	  +b2*2718.28757443698163d0
      	  ia=a
      	  a=a-ia
      	  b(j2)=1.d0-a
      	  a=a+b1+b2+0.706937269307672907d0
      	  ia=a
      	  a=a-ia
 1     	endif
      	random=a*eps
	return
	end


**********************************************************************      
c Count # of bits '1' in the 'n' first bits of integer i (<2^n-1)
c result given in ''k'
 
      subroutine popcnt (n,i,k)
      implicit none
      integer n,i,j,k
      
      k=0
      do j=0,n-1
         if (iand(ishft(i,-j),1).eq.1) k=k+1 
      enddo
      end




















