      program susceptibility
      implicit none

      integer n                 ! Lattice size (1D)
      parameter(n=8)            !

      character*100 name,namechi
      integer lname,lnamechi
      integer nstates

      real fond

      real e,f,sz
      dimension e(2**n),f(2**n),sz(2**n)

      integer i,j
      real j2
      parameter (j2=0.5)
      real t,tmin,tmax
      parameter(tmin=0.001)
      parameter(tmax=4.000)
      integer npoints
      parameter(npoints=100)
      real chi
      real z0,z1

      call makename(name,j2,lname,n)
      open(30,file=name(1:lname),status='unknown')

      call findname(namechi,lnamechi,n)
      open(40,file=namechi(1:lnamechi),status='unknown')

      read(30,*) nstates

      if (nstates.ne.(2**n)) then
         print*,'Wrong number of states',nstates
         stop
      endif

      print*,'Looking for Ground state...'

      fond=10000.

      do i=1,nstates
         read(30,*) e(i),sz(i)
      enddo

      close(30)

      do i=1,nstates
         if (e(i).lt.fond) then
            fond=e(i)
         endif
      enddo

      print*,'Ground state energy : ',fond

      do i=1,nstates
         f(i)=e(i)-fond
      enddo
      
      print*,'Computing susceptibility...'

      do i=0,npoints-1
         t=tmin+((tmax-tmin)/(float(npoints-1)))*i
         
         z0=0.
         z1=0.
         
         do j=1,nstates
            z0=z0+exp(-f(j)/t)
            z1=z1+(sz(j)**2)*exp(-f(j)/t)
         enddo                             

         chi=(1/t)*(z1/z0)
         write(40,*) t,chi/float(n)
         print*,t,chi/float(n)

      enddo

      close(40)

      end


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine findname(namechi,lname,n_sites)
      implicit none
      character*100 namechi
      integer lname,n_sites 



      lname=11
      if (n_sites.ge.10) then
         lname=lname+1
         namechi(1:lname)='Chi.N='//char(48+(n_sites/10))//
     &        char(48+n_sites-10*(n_sites/10))//'.dat'
      else
         namechi(1:lname)='Chi.N='//char(48+n_sites)//'.dat'
      endif

      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine makename(name,j2,lname,n_sites)
      implicit none
      character*100 name
      integer lname,lname_temp,n_sites 
      real j2
      character*100 sj2
      integer lsj2

      call re2str(j2,sj2,lsj2)

      lname=16+lsj2
      if (n_sites.ge.10) then
         lname=lname+1
         name(1:lname)='Spectrum.N='//char(48+(n_sites/10))//
     &        char(48+n_sites-10*(n_sites/10))//'.J2='//sj2(1:lsj2)
      else
         name(1:lname)='Spectrum.N='//char(48+n_sites)
     &        //'.J2='//sj2(1:lsj2)
      endif
      
      lname_temp=lname
      lname=lname+4
      name(1:lname)=name(1:lname_temp)//'.dat'


      end


cccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine re2str(num,strg,lstrg)
      real num
      character*100 strg
      integer lstrg
      
      write (strg,'(f5.3)') num; strg=adjustl(strg)
      lstrg=len_trim(strg)
      end
      

